@@ -34,7 +34,7 @@
 #include <linux/timer.h>
 #include <linux/dmi.h>
 #include <linux/jiffies.h>
-#include <linux/thinkpad_ec.h>
+#include "thinkpad_ec.h"
 #include <linux/pci_ids.h>
 #include <linux/version.h>
 
@@ -89,13 +89,11 @@ static struct input_dev *hdaps_idev_raw; /* raw hdaps sensor readouts */
 static unsigned int hdaps_invert = HDAPS_ORIENT_UNDEFINED;
 static int needs_calibration;
 
-
 /* Configuration: */
 static int sampling_rate = 50;       /* Sampling rate  */
 static int oversampling_ratio = 5;   /* Ratio between our sampling rate and
 				      * EC accelerometer sampling rate      */
 static int running_avg_filter_order = 2; /* EC running average filter order */
-static int fake_data_mode;           /* Enable EC fake data mode? */
 
 /* Latest state readout: */
 static int pos_x, pos_y;      /* position */
@@ -238,28 +236,6 @@ static int hdaps_set_power(int on)
 	return 0;
 }
 
-/**
- * hdaps_set_fake_data_mode - enable or disable EC test mode
- * EC test mode fakes accelerometer data using an incrementing counter.
- * Returns zero on success and negative error code on failure.  Can sleep.
- */
-static int hdaps_set_fake_data_mode(int on)
-{
-	struct thinkpad_ec_row args =
-		{ .mask = 0x0007, .val = {0x17, 0x83, on?0x01:0x00} };
-	struct thinkpad_ec_row data = { .mask = 0x8000 };
-	int ret = thinkpad_ec_read_row(&args, &data);
-	if (ret)
-		return ret;
-	if (data.val[0xF] != 0x00) {
-		printk(KERN_WARNING "failed setting hdaps fake data to %d\n",
-		       on);
-		return -EIO;
-	}
-	printk(KERN_DEBUG "hdaps: fake_data_mode set to %d\n", on);
-	return 0;
-}
-
 /**
  * hdaps_set_ec_config - set accelerometer parameters.
  * @ec_rate: embedded controller sampling rate
@@ -399,9 +375,6 @@ static int hdaps_device_init(void)
 				running_avg_filter_order))
 		{ FAILED_INIT("hdaps_set_ec_config failed"); goto bad; }
 
-	if (hdaps_set_fake_data_mode(fake_data_mode))
-		{ FAILED_INIT("hdaps_set_fake_data_mode failed"); goto bad; }
-
 	thinkpad_ec_invalidate();
 	udelay(200);
 
@@ -678,26 +651,6 @@ static ssize_t hdaps_running_avg_filter_order_store(
 	return count;
 }
 
-static ssize_t hdaps_fake_data_mode_store(struct device *dev,
-					  struct device_attribute *attr,
-					  const char *buf, size_t count)
-{
-	int on, ret;
-	if (sscanf(buf, "%d", &on) != 1 || on < 0 || on > 1)
-		return -EINVAL;
-	ret = hdaps_set_fake_data_mode(on);
-	if (ret)
-		return ret;
-	fake_data_mode = on;
-	return count;
-}
-
-static ssize_t hdaps_fake_data_mode_show(
-	struct device *dev, struct device_attribute *attr, char *buf)
-{
-	return sprintf(buf, "%d\n", fake_data_mode);
-}
-
 static int hdaps_mousedev_open(struct input_dev *dev)
 {
 	if (!try_module_get(THIS_MODULE))
@@ -737,8 +690,6 @@ static DEVICE_ATTR(oversampling_ratio, 0644,
 static DEVICE_ATTR(running_avg_filter_order, 0644,
 		   hdaps_running_avg_filter_order_show,
 		   hdaps_running_avg_filter_order_store);
-static DEVICE_ATTR(fake_data_mode, 0644,
-		   hdaps_fake_data_mode_show, hdaps_fake_data_mode_store);
 
 static struct attribute *hdaps_attributes[] = {
 	&dev_attr_position.attr,
@@ -750,7 +701,6 @@ static struct attribute *hdaps_attributes[] = {
 	&dev_attr_sampling_rate.attr,
 	&dev_attr_oversampling_ratio.attr,
 	&dev_attr_running_avg_filter_order.attr,
-	&dev_attr_fake_data_mode.attr,
 	NULL,
 };
 
@@ -764,9 +714,9 @@ static struct attribute_group hdaps_attribute_group = {
 /* hdaps_dmi_match_invert - found an inverted match. */
 static int __init hdaps_dmi_match_invert(const struct dmi_system_id *id)
 {
-	int orient = (int) id->driver_data;
+	unsigned int orient = (kernel_ulong_t) id->driver_data;
 	hdaps_invert = orient;
-	printk(KERN_INFO "hdaps: %s detected, setting orientation %d\n",
+	printk(KERN_INFO "hdaps: %s detected, setting orientation %u\n",
 	       id->ident, orient);
 	return 1; /* stop enumeration */
 }
@@ -800,6 +750,15 @@ struct dmi_system_id __initdata hdaps_whitelist[] = {
 	HDAPS_DMI_MATCH_INVERT("LENOVO", "ThinkPad X60s", HDAPS_ORIENT_INVERT_Y),
 	HDAPS_DMI_MATCH_INVERT("LENOVO", "ThinkPad X60", HDAPS_ORIENT_SWAP | HDAPS_ORIENT_INVERT_X),
 	HDAPS_DMI_MATCH_INVERT("LENOVO", "ThinkPad X61", HDAPS_ORIENT_SWAP | HDAPS_ORIENT_INVERT_X),
+	HDAPS_DMI_MATCH_INVERT("LENOVO", "ThinkPad T400s", HDAPS_ORIENT_INVERT_X),
+	HDAPS_DMI_MATCH_INVERT("LENOVO", "ThinkPad T400", HDAPS_ORIENT_INVERT_XY),
+	HDAPS_DMI_MATCH_INVERT("LENOVO", "ThinkPad T410", HDAPS_ORIENT_INVERT_XY),
+	HDAPS_DMI_MATCH_INVERT("LENOVO", "ThinkPad T500", HDAPS_ORIENT_INVERT_XY),
+	HDAPS_DMI_MATCH_INVERT("LENOVO", "ThinkPad X200", HDAPS_ORIENT_SWAP | HDAPS_ORIENT_INVERT_X | HDAPS_ORIENT_INVERT_Y),
+	HDAPS_DMI_MATCH_INVERT("LENOVO", "ThinkPad X201 Tablet", HDAPS_ORIENT_SWAP | HDAPS_ORIENT_INVERT_XY),
+	HDAPS_DMI_MATCH_INVERT("LENOVO", "ThinkPad X201s", HDAPS_ORIENT_SWAP | HDAPS_ORIENT_INVERT_XY),
+	HDAPS_DMI_MATCH_INVERT("LENOVO", "ThinkPad X201", HDAPS_ORIENT_SWAP | HDAPS_ORIENT_INVERT_X),
+	HDAPS_DMI_MATCH_INVERT("LENOVO", "ThinkPad X220", HDAPS_ORIENT_SWAP),
 	{ .ident = NULL }
 };
 
