@@ -42,12 +42,12 @@
 #include <linux/mc146818rtc.h>	/* CMOS defines */
 #include <linux/delay.h>
 #include <linux/version.h>
-#include <linux/thinkpad_ec.h>
+#include "thinkpad_ec.h"
 #include <linux/platform_device.h>
 #include <asm/uaccess.h>
 #include <asm/io.h>
 
-#define TP_VERSION "0.40"
+#define TP_VERSION "0.41"
 #define TP_DESC "ThinkPad SMAPI Support"
 #define TP_DIR "smapi"
 
@@ -88,7 +88,7 @@ MODULE_PARM_DESC(debug, "Debug level (0=off, 1=on)");
 static struct { u8 rc; char *msg; int ret; } smapi_retcode[] =
 {
 	{0x00, "OK", 0},
-	{0x53, "SMAPI fuction is not available", -ENXIO},
+	{0x53, "SMAPI function is not available", -ENXIO},
 	{0x81, "Invalid parameter", -EINVAL},
 	{0x86, "Function is not supported by SMAPI BIOS", -EOPNOTSUPP},
 	{0x90, "System error", -EIO},
@@ -109,7 +109,11 @@ static struct { u8 rc; char *msg; int ret; } smapi_retcode[] =
 #define SMAPI_PORT2 0x4F           /* fixed port, meaning unclear */
 static unsigned short smapi_port;  /* APM control port, normally 0xB2 */
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,37)
 static DECLARE_MUTEX(smapi_mutex);
+#else
+static DEFINE_SEMAPHORE(smapi_mutex);
+#endif
 
 /**
  * find_smapi_port - read SMAPI port from NVRAM
@@ -775,7 +779,7 @@ static ssize_t store_battery_start_charge_thresh(struct device *dev,
 
 	down(&smapi_mutex);
 	ret = get_thresh(bat, THRESH_STOP, &other_thresh);
-	if (ret != -EOPNOTSUPP) {
+	if (ret != -EOPNOTSUPP && ret != -ENXIO) {
 		if (ret) /* other threshold is set? */
 			goto out;
 		ret = get_real_thresh(bat, THRESH_START, NULL);
@@ -817,7 +821,7 @@ static ssize_t store_battery_stop_charge_thresh(struct device *dev,
 
 	down(&smapi_mutex);
 	ret = get_thresh(bat, THRESH_START, &other_thresh);
-	if (ret != -EOPNOTSUPP) { /* other threshold exists? */
+	if (ret != -EOPNOTSUPP && ret != -ENXIO) { /* other threshold exists? */
 		if (ret)
 			goto out;
 		/* this threshold exists? */
